// HexComPtr template class
// version 2002-08-21
// 
// This class is derived from _com_ptr_t to implement
// IClassFactory2 licensing functions.
//
// Usage example:
// HexComPtr< HEX_IIID( IConnectionSync ) > pDns;
// HRESULT hr = pDns.CreateInstanceLic( __uuidof( Connection ), L"runtime license key" );
//
// History:
// 2002-08-21  Created, based on early rough version
//
// Copyright 2002 Hexillion Technologies. All rights reserved.
// 
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.

#pragma once

#include <comdef.h>

template< typename _IIID >
class HexComPtr : public _com_ptr_t< _IIID >
	{
	public:
		HRESULT CreateInstanceLic( const CLSID& rclsid,
								   _bstr_t bstrKey,
								   IUnknown* pOuter=NULL,
								   DWORD dwClsContext = CLSCTX_ALL )
			{
			HRESULT hr;
			IClassFactory2* pCF;

			hr = ::CoGetClassObject( rclsid,
									 dwClsContext,
									 NULL,
									 IID_IClassFactory2,
									 (void**)&pCF );
			
			if( FAILED( hr ) )
				return hr;

			hr = pCF->CreateInstanceLic( pOuter,
										 NULL,
										 GetIID(),
										 bstrKey,
										 reinterpret_cast< void** >( operator&() ) );
			pCF->Release();
			return hr;
			}


		static HRESULT RequestLicKey( const CLSID& rclsid,
			                          _bstr_t& bstrKey,
									  DWORD dwClsContext = CLSCTX_ALL )
			{
			HRESULT hr;
			IClassFactory2* pCF;

			hr = ::CoGetClassObject( rclsid,
									 dwClsContext,
									 NULL,
									 IID_IClassFactory2,
									 (void**)&pCF );

			if( FAILED( hr ) )
				return hr;

			BSTR bstr;
			hr = pCF->RequestLicKey( 0, &bstr );

			if( !FAILED( hr ) )
				bstrKey = _bstr_t( bstr, false );

			pCF->Release();
			return hr;
			}
	};


#define HEX_IIID( Interface ) \
	_com_IIID< Interface, &__uuidof( Interface ) >
